const
	gulp         = require( 'gulp' ),
	browserSync  = require( 'browser-sync' ),
	plumber      = require( 'gulp-plumber' ),
	sass         = require( 'gulp-sass' ),
	less         = require( 'gulp-less' ),
	pug          = require( 'gulp-pug' ),
	jade         = require( 'gulp-jade' ),
	babel        = require( 'gulp-babel' ),
	sourcemaps   = require( 'gulp-sourcemaps' ),
	autoprefixer = require( 'gulp-autoprefixer' ),
	emitty       = require( 'emitty' ).setup( 'dev', 'pug' ), // Hardcode
	lessToScss   = require( 'gulp-less-to-scss' ),
	gulpIf       = require( 'gulp-if' ),
	validator    = require( 'gulp-html-validator' ),
	insert       = require( 'gulp-insert' ),
	concat       = require( 'gulp-concat' ),
	rename       = require( 'gulp-rename' ),
	cache        = require( 'gulp-cache' ),
	Balloon      = require( 'node-notifier' ).WindowsBalloon,
	notifier     = new Balloon();

module.exports = {
	sass( end ) {
		let fail = false, startTime = process.hrtime(), util = require( './util.js' );
		return gulp.src( global.config.sass.source )
			.pipe( plumber({ errorHandler: util.defaultErrorHandler }) )
			.pipe( sourcemaps.init({ loadMaps: true, largeFile: true, identityMap: true }) )
			.pipe( sass( global.config.sass.options ) )
			.on( 'error', function() {
				fail = true;
				end();
			} )
			.pipe( gulpIf( global.config.autoprefixer.enable, autoprefixer( global.config.autoprefixer.options ) ) )
			.pipe( sourcemaps.write( './' ) )
			.pipe( gulp.dest( global.config.sass.dest ) )
			.on( 'end', function() { if( !fail ) {
				browserSync.reload('*.css');
				notifier.notify({ title: 'SASS', message: `Successfully compiled!\r${ util.runtime( startTime ) }`, time: 3000 });
			}})
	},

	less() {
		let fail = false, startTime = process.hrtime(), util = require( './util.js' );
		return gulp.src( global.config.less.source )
			.pipe( plumber({ errorHandler: util.defaultErrorHandler }) )
			.pipe( sourcemaps.init({ loadMaps: true, largeFile: true, identityMap: true }) )
			.pipe( less() )
			.on( 'error', function() { fail = true } )
			.pipe( gulpIf( global.config.autoprefixer.enable, autoprefixer( global.config.autoprefixer.options ) ) )
			.pipe( sourcemaps.write( './' ) )
			.pipe( gulp.dest( global.config.less.dest ) )
			.on( 'end', function() { if( !fail ) {
				browserSync.reload('*.css');
				notifier.notify({ title: 'LESS', message: `Successfully compiled!\r${ util.runtime( startTime ) }`, time: 3000 });
			}})
	},

	pug() {
		return new Promise( function( resolve, reject ) {
			let
				fail = false,
				startTime = process.hrtime(),
				util = require( './util.js' ),
				main = function () {
					gulp.src( global.config.pug.source )
						.pipe( plumber({ errorHandler: util.defaultErrorHandler }) )
						.pipe( gulpIf( global.watch && global.config.pug.options.emitty, emitty.filter( global.emittyChangedFile ) ) )
						.pipe( pug( global.config.pug.options ) )
						.on( 'error', function() { fail = true } )
						.pipe( gulp.dest( global.config.pug.dest ) )
						.on( 'end', function() {
							if( !fail ) {
								browserSync.reload();
								notifier.notify({ title: 'PUG', message: `Successfully compiled!\r${ util.runtime( startTime ) }`, time: 3000 });
							}
							resolve();
						});
				};

			if ( global.config.pug.options.emitty ) emitty.scan( global.emittyChangedFile ).then( main );
			else main();
		});
	},

	jade() {
		let fail = false, startTime = process.hrtime(), util = require( './util.js' );
		return gulp.src( global.config.jade.source )
			.pipe( plumber({ errorHandler: util.defaultErrorHandler }) )
			.pipe( jade( global.config.jade.options ) )
			.on( 'error', function() { fail = true } )
			.pipe( gulp.dest( global.config.jade.dest ) )
			.on( 'end', function() { if( !fail ) {
				browserSync.reload();
				notifier.notify({ title: 'JADE', message: `Successfully compiled!\r${ util.runtime( startTime ) }`, time: 3000 });
			}})
	},

	babel() {
		let fail = false, startTime = process.hrtime(), util = require( './util.js' );
		return gulp.src( global.config.babel.source )
			.pipe( plumber({ errorHandler: util.defaultErrorHandler }) )
			.pipe( babel( global.config.babel.options ) )
			.on( 'error', function() { fail = true } )
			.pipe( gulp.dest( global.config.babel.dest ) )
			.on( 'end', function() { if( !fail ) {
				browserSync.reload();
				notifier.notify({ title: 'JS', message: `Successfully compiled!\r${ util.runtime( startTime ) }`, time: 3000 });
			}})
	},

	htmlValidate() {
		let startTime = process.hrtime(), util = require( './util.js' );
		return gulp.src( global.config.htmlValidate.source )
			.pipe( plumber({ errorHandler: util.defaultErrorHandler }) )
			.pipe( validator({ format: 'text' }) )
			.pipe( insert.transform( function( contents, file ) {
				return `## ${file.path}\n${contents}`;
			}))
			.pipe( concat({ path: '#report.md' }) )
			.pipe( gulp.dest( global.config.htmlValidate.report ) )
			.on( 'end', function() {
				notifier.notify({ title: 'Validation Complete', message: util.runtime( startTime ), time: 3000 });
			})
	},

	jadeToPug() {
		let startTime = process.hrtime(), util = require( './util.js' );
		return gulp.src( global.config.jadeToPug.source )
			.pipe( plumber({ errorHandler: util.defaultErrorHandler }) )
			.pipe( rename( function( path ) { path.extname = ".pug"; } ) )
			.pipe( gulp.dest( global.config.jadeToPug.dest ) )
			.on( 'end', function() {
				notifier.notify({ title: 'Conversion completed', message: util.runtime( startTime ), time: 3000 });
			})
	},

	lessToScss() {
		let startTime = process.hrtime(), util = require( './util.js' );
		return gulp.src( global.config.lessToScss.source )
			.pipe( plumber({ errorHandler: util.defaultErrorHandler }) )
			.pipe( lessToScss() )
			.pipe( gulp.dest( global.config.lessToScss.dest ) )
			.on( 'end', function() {
				notifier.notify({ title: 'Conversion completed', message: util.runtime( startTime ), time: 3000 });
			})
	},

	cache() {
		return cache.clearAll();
	}
};
